#include "channel.h"

Channel::Channel(int samplerate, int channels, int samplesize) : _samplerate(samplerate), _channels(channels), _samplesize(samplesize) {
}

void Channel::write(char *path) const {
	FILE *f = fopen(path, "wb");
	for (const_iterator it = begin(); it != end(); it++) {
		(*it)->write(f);
	}
	fclose(f);
}

void Channel::ltrim(double value) {
	double size = 0;
	while (!isEmpty()) {
		Samples *samples = first();
		size += samples->nsamples();
		if (duration(size) > value)
			break;
		removeFirst();
		delete samples;
	}
}

void Channel::rtrim(double value) {
	double size = 0;
	while (!isEmpty()) {
		Samples *samples = last();
		size += samples->nsamples();
		if (duration(size) > value)
			break;
		removeLast();
		delete samples;
	}
}
/*
int Channel::endMax(int nbuffers) const {
	int max = 0;

	const_iterator it = end();
	it--;

	for (int i = 0; i < nbuffers; i++, it--) {
		int absmax = (*it)->absmax();
		max = MAX(max, absmax);
		if (it == begin())
			break;
	}
	return max;
}
*/

void Channel::clear() {
	for (iterator it = begin(); it != end(); it++) {
		delete (*it);
	}
	ChannelList::clear();
}

size_t Channel::nsamples() const {
	size_t result = 0;
	for (const_iterator it = begin(); it != end(); it++) {
		result += (*it)->nsamples();
	}
	return result;
}

size_t Channel::pos(Samples *samples) {
	if (!samples)
		return -1;

	size_t nsamples = 0;
	for (iterator it = begin(); it != end(); it++) {
		if (samples == *it)
			return nsamples; 
		nsamples += samples->nsamples();
	}
	return -1;
}

double Channel::posT(Samples *samples) {
	size_t nsamples = pos(samples);
	if (nsamples == (size_t) -1)
		return 0;

	return duration(nsamples);
}

int Channel::channels() const {
	return _channels;
}

void Channel::setChannels(int value) {
	_channels = value;
}

int Channel::samplesize() const {
	return _samplesize;
}

void Channel::setSamplesize(int value) {
	_samplesize = value;
}

int Channel::samplerate() const {
	return _samplerate;
}

void Channel::setSamplerate(int value) {
	_samplerate = value;
}

double Channel::duration(int nsamples) const {
	return (double) nsamples / (double) _samplerate;
}

double Channel::duration() const {
	return duration(nsamples());
}

double Channel::duration(Samples *samples) const {
	return duration(samples->nsamples());
}

void Channel::mul(double value) {
	for (iterator it = begin(); it != end(); it++) {
		(*it)->mul(value);
	}
}

void Channel::limits(int &min, int &max) const {
	min = 0;
	max = 0;
	for (const_iterator it = begin(); it != end(); it++) {
		int _min, _max;
		(*it)->limits(_min, _max);
		min = MIN(_min, min);
		max = MAX(_max, max);
	}
}

int Channel::absmax() const {
	int min, max;
	limits(min, max);
	min = abs(min);
	max = abs(max);
	return MAX(min, max);
}

int Channel::maxval() const {
	Samples *samples = first();
	return (samples) ? samples->maxval() : -1;
}

