#include "codec_flac.h"

#include <QDebug>

#include <cstdlib>

DecoderFlac::DecoderFlac() {
	set_md5_checking(true);
}

void DecoderFlac::error_callback(FLAC__StreamDecoderErrorStatus status) {
	qDebug() << "Got error callback: " << FLAC__StreamDecoderErrorStatusString[status];
}

FLAC__StreamDecoderWriteStatus DecoderFlac::write_callback(const FLAC__Frame *frame, const FLAC__int32 *const /*data*/[]) {
	//TODO: manage non 16 bits streams
	if (frame->header.bits_per_sample != 16) {
		return FLAC__STREAM_DECODER_WRITE_STATUS_ABORT;
	}
//	audiobuffer_ptr->serialize((int32_t const **) data, frame->header.blocksize);
	return FLAC__STREAM_DECODER_WRITE_STATUS_CONTINUE;
}

void DecoderFlac::metadata_callback(const FLAC__StreamMetadata */*metadata*/) {
//	audiobuffer_ptr->params.rate = metadata->data.stream_info.sample_rate;
//	audiobuffer_ptr->params.channels = metadata->data.stream_info.channels;
}

bool DecoderFlac::open(std::string const path) {
	err = init(path);
	return (err == FLAC__STREAM_DECODER_INIT_STATUS_OK);
}

bool DecoderFlac::open(FILE *file) {
	err = init(file);
	return (err == FLAC__STREAM_DECODER_INIT_STATUS_OK);
}

bool DecoderFlac::decode(Channel &/*channel*/) {
	if (err != FLAC__STREAM_DECODER_INIT_STATUS_OK) {
		qDebug() << "ERROR: initializing decoder: %s\n" << FLAC__StreamDecoderInitStatusString[err];
		return false;
	}

//	audiobuffer_ptr = &audiobuffer;
	return process_until_end_of_stream();
}

bool EncoderFlac::open(QString const path) {
	init_status = FLAC::Encoder::File::init(path.toStdString());
	return (init_status == FLAC__STREAM_ENCODER_INIT_STATUS_OK);
}

bool EncoderFlac::open(FILE *file) {
	init_status = FLAC::Encoder::File::init(file);
	return (init_status == FLAC__STREAM_ENCODER_INIT_STATUS_OK);
}

bool EncoderFlac::init(Channel &channel) {
	bool result = set_verify(true);
	result &= set_compression_level(5);
	result &= set_channels(channel.channels());
	result &= set_bits_per_sample(channel.samplesize() * 8);
	result &= set_sample_rate(channel.samplerate());
	result &= set_total_samples_estimate(channel.nsamples());
	return result;
}

bool EncoderFlac::setTag(Tag const &tag) {
	metadata[0] = FLAC__metadata_object_new(FLAC__METADATA_TYPE_VORBIS_COMMENT);

	FLAC__StreamMetadata_VorbisComment_Entry entry;
	for(Tag::const_iterator it = tag.begin(); it != tag.end(); it++) {
		FLAC__metadata_object_vorbiscomment_entry_from_name_value_pair(&entry, it->name.toUpper().toUtf8(), it->value.toUtf8());
		FLAC__metadata_object_vorbiscomment_append_comment(metadata[0], entry, true);
	}

	metadata[1] = FLAC__metadata_object_new(FLAC__METADATA_TYPE_PADDING);
	metadata[1]->length = 1024 * 4;

	return set_metadata(metadata, 1);
}

bool EncoderFlac::encode(Channel &channel) {
	if (init_status != FLAC__STREAM_ENCODER_INIT_STATUS_OK) {
		return false;
	}

	for (Channel::const_iterator it = channel.begin(); it != channel.end(); it++) {
		Samples *samples = *it;

		TSamples<int32_t> buffer;
		buffer.setCapacity(samples->nsamples());
		buffer.append(*samples);
		if (!process_interleaved((const FLAC__int32 *) buffer.data(), buffer.nsamples()))
			return false;
	}
	return finish();
}

EncoderFlac::EncoderFlac() {
	init_status = FLAC__STREAM_ENCODER_INIT_STATUS_ENCODER_ERROR;
};
