#ifndef _RECORDER_CUTTER_H
#define _RECORDER_CUTTER_H

#include <QList>
#include <QObject>
#include <QMutex>

#include "recorder.h"
#include "channel.h"

class Cutter : public Recorder {
protected:
	Channel channel;
	QMutex mutex;
	bool terminated;
	bool started;
	bool saturated;
	double silence;
	Samples *lastSilenceWaiting;
	Samples *firstSilenceEnding;

	double thresholdStart;
	double thresholdSilence;
	double thresholdSaturation;
	double bufferLength;
	double marginBefore;
	double marginAfter;
	double stopSilence;
	double minimalLength;
	bool normalize;

	virtual void terminate() = 0;
	virtual void _save() = 0;

private:
	void _clear();
	void _normalize();
	void _rtrim(int nbuffers);
	bool _isValid();
	void _addSamplesWaiting(Samples *samples, double level);
	void _addSamplesEnding(Samples *samples, double level);
	void _addSamples(Samples *samples);

protected:
	double _recordFrom();
	double _recordTo();

	void clear();
	void addSamples(Samples *samples);

public:
	bool begin();
	bool end();

	double getThresholdStart() const;
	void setThresholdStart(double value);
	double getThresholdSilence() const;
	void setThresholdSilence(double value);
	double getThresholdSaturation() const;
	void setThresholdSaturation(double value);
	double getMarginBefore() const;
	void setMarginBefore(double value);
	double getMarginAfter() const;
	void setMarginAfter(double value);
	double getStopSilence() const;
	void setStopSilence(double value);
	bool getNormalize() const;
	void setNormalize(bool value);

	Cutter();
	~Cutter();
};

#endif
