#include "file.h"

#include <QMessageBox>
#include <QDebug>
#include <QXmlStreamWriter>

void FileItems::debug() const {
/*
	for (const_iterator it = begin(); it != end(); it++) {
		RecorderItemData *item = *it;	
		item->debug();
	}
*/
}

void FileItems::writeXml(QXmlStreamWriter &stream) const {
	for (const_iterator it = begin(); it != end(); it++) {
		RecorderItemData *item = *it;	
		stream.writeStartElement("item");
		stream.writeAttribute("text", item->text());
		if (!item->filename().isNull()) stream.writeAttribute("filename", item->filename());
		if (!item->fields().isNull()) stream.writeAttribute("fields", item->fields());
		stream.writeEndElement();
	}
}

bool File::readXml(QXmlInputSource *source) {
	FileHandler handler(this);
	QXmlSimpleReader reader;
	reader.setContentHandler(&handler);
	reader.setErrorHandler(&handler);

	if (!reader.parse(source))
        	return false;

	return true;
}

bool File::read(QString path) {
	QFile file(path);
	QXmlInputSource source(&file);
	return readXml(&source);
}

void File::debug() const {
	tag.debug();
	items.debug();
}

void File::write(QXmlStreamWriter &stream) const {
	stream.setAutoFormatting(true);
//	stream.autoFormattingIndent(-1);
	stream.writeStartDocument();
	stream.writeStartElement("session");

	stream.writeStartElement("tag");
	tag.writeXmlAttributes(stream);	
	stream.writeEndElement();

	stream.writeStartElement("items");
	items.writeXml(stream);	
	stream.writeEndElement();

	stream.writeEndElement();
	stream.writeEndDocument();
}

void File::write(QString fileName) const {
	QFile data(fileName);
	if (data.open(QFile::WriteOnly | QFile::Text)) {
		QXmlStreamWriter stream(&data);
		write(stream);
	}
}

void File::clear() {
	for (FileItems::iterator it = items.begin(); it != items.end(); it++) {
		RecorderItemData *item = *it;
		delete item;
	}
	items.clear();
}

File::~File() {
	clear();
}

FileHandler::FileHandler(File *_file) : file(_file) {
}

bool FileHandler::startElement(const QString &, const QString &, const QString &qName, const QXmlAttributes &attributes) {
	if (qName == "session") {
		return true;
	}
	else if (qName == "tag") {
		file->tag.readXmlAttributes(attributes);
		return true;
	}
	else if (qName == "items") {
		return true;
	}
	else if (qName == "item") {
		QString text = attributes.value("text");
		QString filename = attributes.value("filename");
		QString fields = attributes.value("fields");
		if (!text.isEmpty()) {
			RecorderItemData *item = new RecorderItemData(text, filename, fields);
			file->items.push_back(item);
		}
		return true;
	}
	return false;
}

bool FileHandler::fatalError(const QXmlParseException &exception) {
	QMessageBox::information(NULL, QObject::tr("Error"),
	QObject::tr("Parse error at line %1, column %2:\n%3")
		.arg(exception.lineNumber())
		.arg(exception.columnNumber())
		.arg(exception.message()));
	return false;
}


