/*
 *   Copyright (C) 2010 Association Shtooka <dev-public@shtooka.net>
 *
 *   This file is part of swac-record.
 * 
 *   Swac-record is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *   
 *   Swac-record is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   
 *   You should have received a copy of the GNU General Public License
 *   along with Swac-record.  If not, see <http://www.gnu.org/licenses/>.
 */

#define _MODEL_ITEMS_CPP

#include "model_items.h"

#include <QXmlInputSource>
#include <QXmlSimpleReader>

ModelItem::ModelItem(QString _value, QString _name) {
	value = _value;
	name = _name;
}

int ModelItems::rowCount(const QModelIndex &) const {
	return items.count();
}

QVariant ModelItems::data(const QModelIndex &index, int role) const {
	int row = index.row();
	if (row >= items.count() or row < 0) return QVariant();
	ModelItem *item = items[row];

	if (role == Qt::DisplayRole) {
		return item->value + (item->name.isEmpty() ? "" : " (" + item->name + ")");
	}
	if (role == Qt::UserRole) {
		return item->value;
	}
	return QVariant();
}

void ModelItems::add(QString value, QString name) {
	items.push_back(new ModelItem(value, name));
}

ModelItems::ModelItems() {
}

void ModelItems::parseXml(QString const path) {
	ModelItemsParser handler(this);
	QXmlSimpleReader reader;
	reader.setContentHandler(&handler);
	QFile file(path);
	if (!file.open(QFile::ReadOnly | QFile::Text))
		return;

	QXmlInputSource xmlInputSource(&file);
	reader.parse(xmlInputSource);
}

ModelItemsParser::ModelItemsParser(ModelItems *_modelItems) : modelItems(_modelItems) {
}

bool ModelItemsParser::startElement(const QString&, const QString&, const QString &qName, const QXmlAttributes &attributes) {
	if (qName == "item") {
		QString value = attributes.value("value");
		QString name = attributes.value("name");
		if (value != "" and name != "") {
			modelItems->add(value, name);
		}
	}
	return true;
}

QString ItemsComboBox::currentValue() const {
	return itemData(currentIndex(), Qt::UserRole).toString();
}

void ItemsComboBox::setValue(QString value) {
	int id = (value.isNull()) ? 0 : findData(value, Qt::UserRole, Qt::MatchExactly); 
	setCurrentIndex(id >= 0 ? id : 0);
}
