#include "recorder.h"

#include <stdlib.h>
#include <stdint.h>
#include <QDebug>

bool Recorder::init() {
	bool result = (Pa_Initialize() == paNoError);
	device = Pa_GetDefaultInputDevice();
	return result;
}

void Recorder::info() {
	qDebug() << "Lib PortAudio version: " << Pa_GetVersion() << Pa_GetVersionText();
	int c = Pa_GetDeviceCount();
	int defaultInput = Pa_GetDefaultInputDevice();
	for (int i = 0; i < c; i++) {
		qDebug() << "Device #" << i << "=============";
		if (i == defaultInput)
			qDebug() << "[Default]"; 
		const PaDeviceInfo *info = Pa_GetDeviceInfo(i);
		qDebug() << info->name;
		qDebug() << "Default samplerate: " << info->defaultSampleRate;
	}
}

QString Recorder::deviceName(int i) {
	const PaDeviceInfo *info = Pa_GetDeviceInfo(i);
	return info != NULL ? info->name : "";
}

int Recorder::deviceCount() {
	return Pa_GetDeviceCount();
}

void Recorder::setDevice(int value) {
	device = value;
}

int Recorder::getDevice() const {
	return device;
}

bool Recorder::setup() {
	if (device >= Pa_GetDeviceCount()) device = Pa_GetDefaultInputDevice();
	inputParameters.device = device;
	if (inputParameters.device == paNoDevice) {
		return false;
	}

	inputParameters.channelCount = 1;
	inputParameters.sampleFormat = paInt16;

	const PaDeviceInfo *info = Pa_GetDeviceInfo(device);
	if (info) inputParameters.suggestedLatency = info->defaultLowInputLatency;

	inputParameters.hostApiSpecificStreamInfo = NULL;
	return true;
}

bool Recorder::begin() {
	int err = Pa_OpenStream(
		&stream,
		&inputParameters,
		NULL,                 //&outputParameters
		(double) samplerate,
		samplerate / 20,
		paClipOff | paDitherOff,
		Recorder::static_callback,
		this
	);
	return (!err and Pa_StartStream(stream) == paNoError);
}

bool Recorder::end() {
    return (Pa_CloseStream(stream) == paNoError);
}

bool Recorder::isActive() {
	if (!stream) return false;
	return (Pa_IsStreamActive(stream) == 1); 
}

void Recorder::addSamples(Samples *samples) {
	samples->debug();
	delete samples;
}

unsigned int Recorder::getSamplerate() const {
	return samplerate;
}

bool Recorder::setSamplerate(unsigned int value) {
	samplerate = value;
	return true;
}

int Recorder::callback(const void *inputBuffer, void *, unsigned long framesPerBuffer, const PaStreamCallbackTimeInfo*, PaStreamCallbackFlags) {
	if (!inputBuffer) return paComplete;
	TSamples<int16_t> *samples = new TSamples<int16_t>;
	samples->setCapacity(framesPerBuffer);
	samples->append((int16_t *) inputBuffer, framesPerBuffer);
	addSamples(samples);
	return paContinue;
}

int Recorder::static_callback(const void *inputBuffer, void *outputBuffer, unsigned long framesPerBuffer, const PaStreamCallbackTimeInfo* timeInfo, PaStreamCallbackFlags statusFlags, void *userData) {
	Recorder *_this = (Recorder *) userData;
	return _this->callback(inputBuffer, outputBuffer, framesPerBuffer, timeInfo, statusFlags);
}

Recorder::Recorder() : stream(NULL), device(0), samplerate(44100) {
}
