#ifndef _RECORDER_ITEM_H
#define _RECORDER_ITEM_H

#include <QFrame>
#include <QString>
#include <QLineEdit>

#include "tag.h"

class RecorderWidget;
class RecorderItemEditor;

class RecorderItemData {
protected:
	QString _text;
	QString _filename;
	QString _fields;

public:
	QString text() const;
	void setText(QString const value);
	QString filename() const;
	void setFilename(QString const value);
	QString fields() const;
	void setFields(QString const value);
	void toTag(Tag &tag) const;

	RecorderItemData(RecorderItemData const *value);
	RecorderItemData(QString text = "", QString filename = NULL, QString fields = NULL);
};

class RecorderItem : public QWidget, public RecorderItemData {
	Q_OBJECT

private:
	RecorderWidget *widget;
	RecorderItemEditor *editor;
	bool selected;

protected:
	void paintEvent(QPaintEvent *event);
	void mousePressEvent(QMouseEvent *event);

	friend class RecorderItemEditor;

public:
	QSize sizeHint() const;
	void setText(QString const value);
	void setSelected(bool value);
	void edit();
	bool sndExists();

	RecorderItem(RecorderWidget *widget, RecorderItemEditor *editor, QString const text = NULL);
	RecorderItem(RecorderWidget *widget, RecorderItemEditor *editor, RecorderItemData const *data);
};

class RecorderItemEditor : public QLineEdit {
private:
	RecorderItem *item;
	void activate(RecorderItem *item);
	void hide();

	friend class RecorderItem;

protected:
	void keyPressEvent(QKeyEvent *event);

public:
	RecorderItemEditor();
};

#endif

