#ifndef _RECORDER_VIEW_H
#define _RECORDER_VIEW_H

#include <QScrollArea>
#include <QTextStream>
#include <QString>
#include <QVector>
#include <QTimer>

#include "recorder_item.h"

class RecorderView;
class RecorderModel;

class RecorderWidget : public QWidget {
	Q_OBJECT

public:
	typedef bool SndExistsCb(void *data, RecorderItem const *item);

private:
	RecorderView *view;
	RecorderItemEditor *editor;
	int selected;
	QTimer timer;
	bool recording;
	SndExistsCb *sndExistsCb;
	void *sndExistsCbData;

protected:
	typedef QVector<RecorderItem *> Items;
	Items items;

	void keyPressEvent(QKeyEvent *event);
	QSize sizeHint() const;
	void resizeEvent(QResizeEvent *event);


signals:
	void selectionChanged(RecorderItem *item);
	void triggered();
	void deleteItem(RecorderItem *item);

public slots:
	void doLayout();

public:
	void doLayoutAsync();
	void addItem(QString text, bool refresh = true);
	void addItem(RecorderItem *item, bool refresh = true);
	void addItem(RecorderItemData *data, bool refresh = true);
	bool validId(int id);
	void scrollPage(RecorderItem *item);
	void setSelected(int id, bool paginate = false);
	void setSelected(RecorderItem *item, bool paginate = false);
	int prevLine(int id);
	int nextLine(int id);
	bool move(int d, bool paginate = false);
	void del(int id);
	void edit(int id);
	void insert(int id);
	void import(QTextStream &stream);
	void toVector(QVector<RecorderItemData *> &vector) const;
	void fromVector(QVector<RecorderItemData *> &vector);
	int count() const;
	void clear();
	RecorderItem *itemSelected();
	void setFont(QFont font);
	bool sndExists(RecorderItem const *item) const;
	void setRecording(bool value);
	bool getRecording() const;

	RecorderWidget(RecorderView *parent, SndExistsCb *sndExistsCb = NULL, void *sndExistsCbData = NULL);
};

class RecorderView : public QScrollArea {
	Q_OBJECT

public:
	RecorderWidget *widget;

	RecorderView(QWidget *parent, RecorderWidget::SndExistsCb *sndExistsCb = NULL, void *sndExistsCbData = NULL);
};

#endif

