#include "samples.h"

#include <QDebug>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>

void Samples::debug() const {
	qDebug() << "nsamples: " << _nsamples << "capacity: " << _capacity;
}

int Samples::absmax() {
	int min, max;
	limits(min, max);
	min = abs(min);
	max = abs(max);
	return MAX(min, max);
}

void Samples::limits(int &min, int &max) {
	if (!_limits) {
		limitsDo(_min, _max);
		_limits = true;
	}
	min = _min;
	max = _max;
}

size_t Samples::nsamples() const {
	return _nsamples;
}

Samples::Samples() : _capacity(0), _nsamples(0), _limits(false), _min(0), _max(0) {
}

Samples::~Samples() {
}

template <typename T>
void *TSamples<T>::data() const {
	return _data;
}

template <typename T>
size_t TSamples<T>::size() const {
	return _nsamples * sizeof(T);
}

template <typename T>
void TSamples<T>::setCapacity(size_t value) {
	if (_data) free(_data);
	_capacity = value;
	_data = (T *) malloc(_capacity * sizeof(T));
}

template <typename T>
int TSamples<T>::get(size_t n) const {
	return _data[n];
}

template <typename T>
void TSamples<T>::set(size_t n, int value) {
	_data[n] = value;
}

template <typename T>
size_t TSamples<T>::append(T const *src, size_t s) {
	size_t n = MIN(s, _capacity - _nsamples);
	memcpy((void *) (_data + _nsamples), (void *) src, n * sizeof(T));
	_nsamples += n;
	return n;
}

template <typename T>
size_t TSamples<T>::append(Samples &src) {
	size_t s = src.nsamples();
	size_t n = MIN(s, _capacity - _nsamples);

	for (size_t i = 0; i < n; i++) {
		_data[_nsamples + i] = src.get(i);
	}
	_nsamples += n;
	return n;
}

template <typename T>
size_t TSamples<T>::append(T const value, size_t s) {
	size_t n = MIN(s, _capacity - _nsamples);
	for (size_t i = 0; i < n; i++)
		_data[_nsamples + i] = value;
	_nsamples += n;
	return n;
}

template <typename T>
void TSamples<T>::limitsDo(int &min, int &max) const {
	min = 0;
	max = 0;
	for (size_t i = 0; i < _nsamples; i++) {
		min = MIN(min, _data[i]);
		max = MAX(max, _data[i]);
	}
}

template <typename T>
void TSamples<T>::mul(double value) {
	for (size_t i = 0; i < _nsamples; i++) {
		_data[i] = (T) ((double) _data[i] * value);
	}
}

template <typename T>
void TSamples<T>::write(FILE *f) const {
	fwrite((void const *) _data, sizeof(T), _nsamples, f);
}

template <>
int TSamples<int16_t>::maxval() const {
	return SHRT_MAX;
}

template <>
int TSamples<int32_t>::maxval() const {
	return INT_MAX;
}

template <typename T>
TSamples<T>::TSamples() : _data(NULL) {
}

template <typename T>
TSamples<T>::~TSamples() {
	if (_data) free(_data);
}

template class TSamples<int16_t>;
template class TSamples<int32_t>;

