#include "setup_audio.h"

#include <QFormLayout>
#include <QLabel>
#include <cmath>

SetupAudio::SetupAudio(CutterWidget *_cutter) : cutter(_cutter) {
	startThreshold = new QSpinBox;
	startThreshold->setSingleStep(1);
	startThreshold->setMinimum(0);
	startThreshold->setMaximum(100);
	startThreshold->setSuffix("%");

	stopThreshold = new QSpinBox;
	stopThreshold->setSingleStep(1);
	stopThreshold->setMinimum(0);
	stopThreshold->setMaximum(100);
	stopThreshold->setSuffix("%");

	saturationThreshold = new QSpinBox;
	saturationThreshold->setSingleStep(1);
	saturationThreshold->setMinimum(0);
	saturationThreshold->setMaximum(100);
	saturationThreshold->setSuffix("%");

	stopSilence = new QDoubleSpinBox;
	stopSilence->setSingleStep(0.05);
	stopSilence->setMinimum(0);
	stopSilence->setSuffix("s");

	marginBefore = new QDoubleSpinBox;
	marginBefore->setSingleStep(0.05);
	marginBefore->setMinimum(0);
	marginBefore->setSuffix("s");

	marginAfter = new QDoubleSpinBox;
	marginAfter->setSingleStep(0.05);
	marginAfter->setMinimum(0);
	marginAfter->setSuffix("s");

	samplerate = new QComboBox(this);
	samplerate->addItem("22050", 22050);
	samplerate->addItem("24000", 24000);
	samplerate->addItem("32000", 32000);
	samplerate->addItem("44100", 44100);
	samplerate->addItem("48000", 48000);
	samplerate->addItem("88200", 88200);
	samplerate->addItem("96000", 96000);
	samplerate->addItem("192000", 192000);

	normalize = new QCheckBox;

	deviceList = new QComboBox(this);
	deviceList->setModel(new DeviceModel(cutter));

	QFormLayout *formLayout = new QFormLayout;
	formLayout->addRow(new QLabel(tr("Device:")), deviceList);
	formLayout->addRow(new QLabel(tr("Samplerate:")), samplerate);
	formLayout->addRow(new QLabel(tr("Saturation threshold:")), saturationThreshold);
	formLayout->addRow(new QLabel(tr("Start threshold:")), startThreshold);
	formLayout->addRow(new QLabel(tr("Silence threshold:")), stopThreshold);
	formLayout->addRow(new QLabel(tr("Stop silence:")), stopSilence);
	formLayout->addRow(new QLabel(tr("Margin before:")), marginBefore);
	formLayout->addRow(new QLabel(tr("Margin after:")), marginAfter);
	formLayout->addRow(new QLabel(tr("Normalize:")), normalize);

	setLayout(formLayout);
}

void SetupAudio::getValues() {
	startThreshold->setValue(ceil(cutter->getThresholdStart() * 100.0));
	stopThreshold->setValue(ceil(cutter->getThresholdSilence() * 100.0));
	saturationThreshold->setValue(ceil(cutter->getThresholdSaturation() * 100.0));
	marginBefore->setValue(cutter->getMarginBefore());
	marginAfter->setValue(cutter->getMarginAfter());
	stopSilence->setValue(cutter->getStopSilence());
	normalize->setChecked(cutter->getNormalize());
	deviceList->setCurrentIndex(cutter->getDevice());

	int i = samplerate->findData(cutter->getSamplerate());
	if (i != -1) samplerate->setCurrentIndex(i);

}

void SetupAudio::setValues() {
	cutter->setThresholdStart(startThreshold->value() / 100.0);
	cutter->setThresholdSilence(stopThreshold->value() / 100.0);
	cutter->setThresholdSaturation(saturationThreshold->value() / 100.0);
	cutter->setMarginBefore(marginBefore->value());
	cutter->setMarginAfter(marginAfter->value());
	cutter->setStopSilence(stopSilence->value());
	cutter->setNormalize(normalize->isChecked());
	cutter->setDevice(deviceList->currentIndex());
	cutter->setSamplerate(samplerate->currentText().toInt());
}

DeviceModel::DeviceModel(Recorder *_recorder) : recorder(_recorder) {
}

int DeviceModel::rowCount(const QModelIndex &) const {
	return recorder->deviceCount();
}

QVariant DeviceModel::data(const QModelIndex &index, int role) const {
	int row = index.row();
	if (role == Qt::DisplayRole) {
		return recorder->deviceName(row);
	}
/*
	else if (role == Qt::BackgroundRole) {
		return Qt::red;
		if (recorder->getDevice() == row)
			return Qt::red;
	}
*/
	return QVariant();
}

