#include "setup_view.h"

#include <QFormLayout>
#include <QLabel>
#include <QFontDialog>
#include <QSettings>

SetupView::SetupView(RecorderView *_recorderView) : recorderView(_recorderView) {
	margin = new QSpinBox;
	margin->setSingleStep(1);
	margin->setMinimum(0);
	margin->setMaximum(50);
	margin->setSuffix("px");

	fontButton = new QPushButton("Lorem Ipsum");
	connect(fontButton, SIGNAL(clicked()), this, SLOT(chooseFont()));

	QFormLayout *formLayout = new QFormLayout;
	formLayout->addRow(new QLabel(tr("Font:")), fontButton);
	formLayout->addRow(new QLabel(tr("Margin:")), margin);

	setLayout(formLayout);

	//Load settings
	QSettings settings;
	QVariant variant = settings.value("font");
	if (!variant.isNull()) {
		QFont font;
		if (font.fromString(variant.toString()))
			recorderView->widget->setFont(font);
	}
}

SetupView::~SetupView() {
	//Save settings
	QSettings settings;
	settings.setValue("font", recorderView->widget->font().toString());
}

void SetupView::setFont(QFont font) {
	fontButton->setFont(font);
	fontButton->setText(tr("%1 (%2pt)").arg(font.family()).arg(font.pointSize()));
}

void SetupView::getValues() {
	setFont(recorderView->widget->font());
}

void SetupView::setValues() {
	recorderView->widget->setFont(fontButton->font());
}

void SetupView::chooseFont() {
	bool ok;
	QFont font = QFontDialog::getFont(&ok, fontButton->font(), this);
	if (ok) setFont(font);
}
