#ifndef _WINDOW_EDIT_H
#define _WINDOW_EDIT_H

#include <QMainWindow>
#include <QStatusBar>
#include <QTabWidget>
#include <QLineEdit>
#include <QDateEdit>
#include <QFormLayout>
#include <QLabel>
#include <QComboBox>
#include <QPlainTextEdit>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QToolBar>
#include <QTableView>
#include <QAction>
#include <QTreeView>
#include <QTreeView>
#include <QPushButton>

#include "model_items.h"
#include "recorder_view.h"
#include "tag.h"
#include "file.h"
#include "cutter_widget.h"
#include "window_setup.h"

class TabSpeaker;
class TabCollection;
class TabRecord;

class WindowEdit : public QMainWindow {
	Q_OBJECT

private:
	//--Widgets
	QStatusBar *statusBar;
	QTabWidget *tabWidget;

	QString currentFileName;

public:
	//--Tabs
	TabSpeaker *tabSpeaker;
	TabCollection *tabCollection;
	TabRecord *tabRecord;

	ModelItems *modelLanguages;
	ModelItems *modelCountries;

public slots:
	void open();
	void new_();
	void saveAs();
	void save();

public:
	void fromFile(File &file);
	void toTag(Tag &tag) const;
	void toTag(Tag &tag, RecorderItem const *item) const;
	void toFile(File &file) const;

	void save(QString path) const;

	WindowEdit();
};

class TabSpeaker : public QWidget {
	Q_OBJECT

private:
	WindowEdit *windowEdit;

	QLineEdit *name;
	ItemsComboBox *nativeLang;
	QLineEdit *yearOfBirth;
	ItemsComboBox *originCountry;
	QLineEdit *originRegion;
	ItemsComboBox *livingCountry;
	QLineEdit *livingTown;
	ItemsComboBox *gender;
	QPlainTextEdit *description;

public:
	void toTag(Tag &tag) const;
	void fromTag(Tag &tag);

	TabSpeaker(WindowEdit *parent);
};

class TabCollection : public QWidget {
	Q_OBJECT

private:
	WindowEdit *windowEdit;

	ItemsComboBox *lang;
	QLineEdit *name;
	QLineEdit *section;
	QLineEdit *authors;
	QLineEdit *license;
	QLineEdit *copyright;
	QPlainTextEdit *description;
	QLineEdit *directory;
	QPushButton *directorySelect;

private:
	static QString getCopyright(QString authors);

public slots:
	void selectDirectory();
	QString getDirectory() const;
	void feedCopyright();

public:
	void toTag(Tag &tag) const;
	void fromTag(Tag &tag);

	TabCollection(WindowEdit *parent);
	~TabCollection();
};

class TabRecord : public QWidget {
	Q_OBJECT

private:
	WindowEdit *windowEdit;
	WindowSetup *windowSetup;

	ItemsComboBox *lang;
	QVBoxLayout *layout;
	QToolBar *toolBar;
	QAction *actionOpen;
	QAction *actionPaste;
	QAction *actionPlay;
	QAction *actionDelete;
	QAction *actionRecord;
	QAction *actionPause;
	QAction *actionSettings;

	CutterWidget *cutter;

public:
	RecorderView *recorderView;

public slots:
	void paste();
	void open();
	void play();
	void deleteItem(RecorderItem *item);
	void deleteRecording();
	void record();
	void pause();
	void terminated();
	void updateRecordState();
	void setup();
	void selectedItemChanged(RecorderItem *item);

private:
	void deleteRecordingDo(RecorderItem *item);
	QAction *newAction(QIcon icon, QString toolTip, QObject *parent, QKeySequence shortcut = QKeySequence());
	void setCutterOuput(RecorderItem const *item);

public:
	QString sndFilename(QString const mask, RecorderItem const *item) const;
	QString sndPath(RecorderItem const *item) const;
	bool sndExists(RecorderItem const *item) const;
	static bool sndExistsCb(void *t, RecorderItem const *item);

	void toTag(Tag &tag) const;
	void fromTag(Tag &tag);

	TabRecord(WindowEdit *parent);
};

#endif
