#include "window_setup.h"

#include <QFormLayout>
#include <QVBoxLayout>
#include <QLabel>
#include <cmath>

WindowSetup::WindowSetup(CutterWidget *cutter, RecorderView *recorderView) {

	setupAudio = new SetupAudio(cutter);
	setupView = new SetupView(recorderView);

	tabWidget = new QTabWidget(this);
	tabWidget->addTab(setupAudio, QIcon(":/images/mic.png"), tr("Audio"));
	tabWidget->addTab(setupView, QIcon(":/images/fonts.png"), tr("Apparence"));
	tabWidget->setIconSize(QSize(24, 24));

	//--Buttons
	buttonOk = new QPushButton(tr("&Ok"));
	buttonCancel = new QPushButton(tr("&Cancel"));
	buttonApply = new QPushButton(tr("&Apply"));

	buttonBox = new QDialogButtonBox(this);
	buttonBox->addButton(buttonOk, QDialogButtonBox::AcceptRole);
	buttonBox->addButton(buttonCancel, QDialogButtonBox::RejectRole);
	buttonBox->addButton(buttonApply, QDialogButtonBox::ApplyRole);

	connect(buttonApply, SIGNAL(clicked()), this, SLOT(apply()));
	connect(buttonBox, SIGNAL(accepted()), this, SLOT(ok()));
	connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));

	QVBoxLayout *layout = new QVBoxLayout(this);
	layout->addWidget(tabWidget);
	layout->addWidget(buttonBox);

	setWindowTitle(tr("Settup"));
	setLayout(layout);

	setupAudio->getValues();
	setupView->getValues();
}

void WindowSetup::apply() {
	setupAudio->setValues();
	setupView->setValues();
}

void WindowSetup::ok() {
	apply();
	accept();
}

